#
# Uebertragungsfunktionen_Theremino.m
#
# Ein Skript, das die exportierten Daten einer LTspice-Simulation einließt,
# Pole und Nullstellen daraus errechnet und die Übertragungsfunktionen
# ermittelt.
# Die als Text exportierten Daten aus Zeit- und Frequenz-Bereich müssen mit
# einem Editor bearbeitet werden, damit sie von Octave als csv-Datei gelesen
# werden können.
# .txt: 1.00000000000000e-01 (-5.01744494891950e+01dB,8.98172256262358e+01°) ...
# .csv: 1.00000000000000e-01,-5.01744494891950e+01,8.98172256262358e+01, ...
#
# In den Diagrammen werden die Ergebnisse von LTspice und Octave sowohl im
# Frequenz- als auch im Zeitbereich gegenübergestellt.
#
clc;
clear all;
close all;
format long;
# pkg load ltfat;
pkg load control;
#
global Vini Vonn Tdel Tris Tfal Tonn Tper;
#
# Einlesen von Zeitbereichsdaten aus einer LTspice-Simulation.
# Ein DC-Offset muss bereits in LTspice herausgerechnet werden.
#
# simout(:,1) = time
# simout(:,2) = V(input)
# simout(:,3) = V(n1)-612.75637mV
# simout(:,4) = V(n2)-612.75637mV
# simout(:,5) = V(n3)-612.75637mV
# simout(:,6) = V(n4)-612.75637mV
# simout(:,7) = V(n5)-2.1318336V
# simout(:,8) = V(n6)-1.483457V
# simout(:,9) = V(output)
# simout(:,10) = V(wav)
#
simout = csvread("PMT_Adapter_ABel_04_ohne_HS.csv");
t1max=length(simout);
#
t2max=2500;
t2(:,1)=[0.0:simout(t1max,1)/(t2max-1):simout(t1max,1)];
fs2=1/(simout(t1max,1)/(t2max-1));
#
fout(:,1)=interp1(simout(:,1),simout(:,8),t2,"spline");
#
# PULSE aus LTspice
#
Vini=0;
Vonn=-1.35;
Tdel=3e-6;
Tris=4e-9;
Tfal=85e-6;
Tonn=0;
Tper=simout(t1max,1);
#
# Vektor des Input-Impuls
#
fin=ffin(t2);
#
figure(1);
plot(10^6*simout(:,1),simout(:,8),"linewidth",1,"r", ...
     10^6*t2,fout,"linewidth",1,"linestyle",":","b", ...
     10^6*simout(:,1),simout(:,10),"linewidth",1,"linestyle","-.","c", ...
     10^6*t2,fin,"linewidth",1,"k");
title('Eingelesene Daten und Interopolation');
xlabel('Zeit [μs]');
ylabel('Amplitude [V]');
legend("mit LTspice simulierter Output-Impuls (nicht äquidistant)", ...
       "auf äquidistante Zeitwerte interpolierter Output-Impuls", ...
       "realer, digitalisierter Output-Impuls", ...
       "Input-Impuls der LTspice-Simulation");
grid
#
# Einlesen der Frequenzbereichsdaten aus einer LTspice-Simulation
#
# bodeout(:,1) = Freq.
# bodeout(:,2) = V(n1) Magnitude
# bodeout(:,3) = V(n1) Phase
# bodeout(:,4) = V(n2) Magnitude
# bodeout(:,5) = V(n2) Phase
# bodeout(:,6) = V(n3) Magnitude
# bodeout(:,7) = V(n3) Phase
# bodeout(:,8) = V(n4) Magnitude
# bodeout(:,9) = V(n4) Phase
# bodeout(:,10) = V(n5) Magnitude
# bodeout(:,11) = V(n5) Phase
# bodeout(:,12) = V(n6) Magnitude
# bodeout(:,13) = V(n6) Phase
# bodeout(:,14) = V(output) Magnitude
# bodeout(:,15) = V(output) Phase
#
bodeout = csvread("PMT_Adapter_ABel_04_ohne_HS_Stability.csv");
bmax=length(bodeout);
freq=bodeout(:,1);
#
# Korrektur von 360°-Sprümgen in der Phase
#
for m=1:7
  difphi=0.0;
  for n=1:bmax-1
    if (bodeout(n+1,2*m+1)-bodeout(n,2*m+1)) > 330
      difphi=360.0;
    endif
    if difphi > 0.0
      bodeout(n+1,2*m+1)=bodeout(n+1,2*m+1)-difphi;
    endif
  endfor
endfor
#
printf("--- aus den LTspice-Werten werden Pole und Nullstellen ermittelt.\n");
#
for m=1:7
  phi=bodeout(:,2*m+1)/90.0-0.5;
  nz(m)=0;
  np(m)=0;
  for o=fix(min(phi)):fix(max(phi))
    printf("m=%d  o=%d  max(phi)=%g  min(phi)=%g\n",m,o, ...
               max(bodeout(:,2*m+1)),min(bodeout(:,2*m+1)));
    for n=1:bmax-1
      if phi(n) < o && o < phi(n+1)
        nz(m)=nz(m)+1;
        zer(m,nz(m))=freq(n)+(o-phi(n))*(freq(n+1)-freq(n))/(phi(n+1)-phi(n));
       printf("Nullstelle %d  n=%d  freq(n)=%g interpol. Frequenzwert=%g\n", ...
                                                  nz(m),n,freq(n),zer(m,nz(m)));
      endif
      if phi(n) > o && o > phi(n+1)
        np(m)=np(m)+1;
        pol(m,np(m))=freq(n)+(o-phi(n))*(freq(n+1)-freq(n))/(phi(n+1)-phi(n));
       printf("Polstelle  %d  n=%d  freq(n)=%g interpol. Frequenzwert=%g\n", ...
                                                  np(m),n,freq(n),pol(m,np(m)));
      endif
    endfor
  endfor
endfor
printf("--- Pole und Nullstellen sind ermittelt.\n");
#
# Zusammenstellung der Übertragungsfunktionen aus Polen und Nullstellen
#
xsys1 = zpk([0],[-pol(1,1)],1);
[mag1, pha1] = bode (xsys1,freq);
xsys2 = zpk([0 -zer(2,1)],[-pol(2,1) -pol(2,2)],1);
[mag2, pha2] = bode (xsys2,freq);
xsys3 = zpk([0],[-pol(3,1) -pol(3,2)],1500);
[mag3, pha3] = bode (xsys3,freq);
xsys4 = zpk([0],[-pol(4,1) -pol(4,2) -pol(4,3)],2300000);
[mag4, pha4] = bode (xsys4,freq);
xsys5 = zpk([0],[-pol(5,1) -pol(5,2) -pol(5,3) -pol(5,4)],7.75*10^13);
[mag5, pha5] = bode (xsys5,freq);
xsys6 = zpk([0],[-pol(6,1) -pol(6,2) -pol(6,3) -pol(6,4) -pol(6,5)],2.4*10^18);
[mag6, pha6] = bode (xsys6,freq);
xsys7 = zpk([0 0], ...
  [-pol(7,1) -pol(7,2) -pol(7,3) -pol(7,4) -pol(7,5) -pol(7,6)],2.4*10^18);
[mag7, pha7] = bode (xsys7,freq);
#
# Plot der Frequenzbereichsdaten
#
fig=figure(2);
semilogx(freq,20*log10(mag1),"linewidth",1,"b", ...
         freq,20*log10(mag2),"linewidth",1,"r", ...
         freq,20*log10(mag3),"linewidth",1,"m", ...
         freq,bodeout(:,2),"linewidth",1,"linestyle",":","b", ...
         freq,bodeout(:,4),"linewidth",1,"linestyle",":","r", ...
         freq,bodeout(:,6),"linewidth",1,"linestyle",":","m")
title('Magnitude');
xlabel('Frequenz [Hz]');
ylabel('Verstärkung [dB]');
legend("bei n1 mit Octave","bei n2 mit Octave","bei n3 mit Octave", ...
       "bei n1 mit LTspice","bei n2 mit LTspice","bei n3 mit LTspice")
grid
#
fig=figure(3);
semilogx(freq,20*log10(mag4),"linewidth",1,"b", ...
         freq,20*log10(mag5),"linewidth",1,"r", ...
         freq,20*log10(mag6),"linewidth",1,"m", ...
         freq,20*log10(mag7),"linewidth",1,"k", ...
         freq,bodeout(:,8),"linewidth",1,"linestyle",":","b", ...
         freq,bodeout(:,10),"linewidth",1,"linestyle",":","r", ...
         freq,bodeout(:,12),"linewidth",1,"linestyle",":","m", ...
         freq,bodeout(:,14),"linewidth",1,"linestyle",":","k")
title('Magnitude');
xlabel('Frequenz [Hz]');
ylabel('Verstärkung [dB]');
legend("bei n4 mit Octave","bei n5 mit Octave","bei n6 mit Octave", ...
       "bei output mit Octave", ...
       "bei n4 mit LTspice","bei n5 mit LTspice","bei n6 mit LTspice", ...
       "bei output mit LTspice");
grid
#
fig=figure(4);
semilogx(freq,pha1/45,"linewidth",1,"b", ...
         freq,pha2/45,"linewidth",1,"r", ...
         freq,pha3/45,"linewidth",1,"m", ...
         freq,bodeout(:,3)/45,"linewidth",1,"linestyle",":","b", ...
         freq,bodeout(:,5)/45,"linewidth",1,"linestyle",":","r", ...
         freq,bodeout(:,7)/45,"linewidth",1,"linestyle",":","m")
title('Phase');
xlabel('Frequenz [Hz]');
ylabel('Phase [Deg/45°]');
legend("bei n1 mit Octave","bei n2 mit Octave","bei n3 mit Octave", ...
       "bei n1 mit LTspice","bei n2 mit LTspice","bei n3 mit LTspice")
grid
#
fig=figure(5);
semilogx(freq,pha4/45,"linewidth",1,"b", ...
         freq,(pha5-180)/45,"linewidth",1,"r", ...
         freq,(pha6-180)/45,"linewidth",1,"m", ...
         freq,(pha7-180)/45,"linewidth",1,"k", ...
         freq,bodeout(:,9)/45,"linewidth",1,"linestyle",":","b", ...
         freq,bodeout(:,11)/45,"linewidth",1,"linestyle",":","r", ...
         freq,bodeout(:,13)/45,"linewidth",1,"linestyle",":","m", ...
         freq,bodeout(:,15)/45,"linewidth",1,"linestyle",":","k")
title('Phase');
xlabel('Frequenz [Hz]');
ylabel('Phase [Deg/45°]');
legend("bei n4 mit Octave","bei n5 mit Octave","bei n6 mit Octave", ...
       "bei output mit Octave", ...
       "bei n4 mit LTspice","bei n5 mit LTspice","bei n6 mit LTspice", ...
       "bei output mit LTspice");
grid
#
fig=figure(6);
bode(xsys1,xsys2,xsys3,xsys4,xsys5,xsys6,xsys7);
#
# Berechnung der Zeitbereichsdaten aus den Übertragungsfunktionen
#
fn1 = lsim(xsys1,fin,t2(t2max));
fn2 = lsim(xsys2,fin,t2(t2max));
fn3 = lsim(xsys3,fin,8*t2(t2max));
fn4 = lsim(xsys4,fin,8*t2(t2max));
fn5 = lsim(xsys5,fin,8*t2(t2max));
fn6 = lsim(xsys6,fin,8*t2(t2max));
fou = lsim(xsys7,fin,8*t2(t2max));
#
# Plt der Zeitbereichsdaten
#
fig=figure(7);
plot(10^6*simout(:,1),simout(:,2),"linewidth",1,"linestyle","--","k", ...
     10^6*simout(:,1),simout(:,3),"linewidth",1,"c", ...
     10^6*simout(:,1),simout(:,4),"linewidth",1,"r", ...
     10^6*simout(:,1),simout(:,7),"linewidth",1,"c", ...
     10^6*simout(:,1),simout(:,8),"linewidth",1,"r", ...
     10^6*simout(:,1),simout(:,9),"linewidth",1,"b", ...
     10^6*t2,fn1,"linewidth",1,"linestyle","-.","c", ...
     10^6*t2,fn2,"linewidth",1,"linestyle","-.","r", ...
     10^6*t2,-fn5,"linewidth",1,"linestyle","-.","c", ...
     10^6*t2,-fn6,"linewidth",1,"linestyle","-.","r", ...
     10^6*t2,-fou,"linewidth",1,"linestyle","-.","b")
title('Systemoutput');
xlabel('Zeit [μs]');
ylabel('Amplitude [V]');
legend("LTspice-Daten input","LTspice-Daten n1","LTspice-Daten n2", ...
       "LTspice-Daten n5","LTspice-Daten n6","LTspice-Daten output", ...
       "Octave-Daten n1","Octave-Daten n2", ...
       "Octave-Daten n5","Octave-Daten n6","Octave-Daten output");
grid
#
fig=figure(8);
plot(10^6*simout(:,1),simout(:,5),"linewidth",1,"r", ...
     10^6*simout(:,1),simout(:,6),"linewidth",1,"b", ...
     10^6*t2,fn3,"linewidth",1,"linestyle","-.","r", ...
     10^6*t2,fn4,"linewidth",1,"linestyle","-.","b")
title('Systemoutput');
xlabel('Zeit [μs]');
ylabel('Amplitude [V]');
legend("LTspice-Daten n3","LTspice-Daten n4", ...
       "Octave-Daten n3","Octave-Daten n4");
grid
#
